package data

import (
	mongo "cloud.o-forge.io/core/oc-lib/dbs/mongo"
	"cloud.o-forge.io/core/oc-lib/models/utils"
)

type DataMongoAccessor struct {
	utils.AbstractAccessor
}

func (dma *DataMongoAccessor) DeleteOne(id string) (utils.DBObject, int, error) {
	return dma.GenericDeleteOne(id, dma)
}

func (dma *DataMongoAccessor) UpdateOne(set utils.DBObject, id string) (utils.DBObject, int, error) {
	return dma.GenericUpdateOne(set, id, dma)
}

func (dma *DataMongoAccessor) StoreOne(data utils.DBObject) (utils.DBObject, int, error) {
	return dma.GenericStoreOne(data, dma)
}

func (dma *DataMongoAccessor) CopyOne(data utils.DBObject) (utils.DBObject, int, error) {
	return dma.GenericStoreOne(data, dma)
}

func (dma *DataMongoAccessor) LoadOne(id string) (utils.DBObject, int, error) {
	var data DataResource
	res_mongo, code, err := mongo.MONGOService.LoadOne(id, dma.GetType())
	if err != nil {
		dma.Logger.Error().Msg("Could not retrieve " + id + " from db. Error: " + err.Error())
		return nil, code, err
	}
	res_mongo.Decode(&data)

	return &data, 200, nil
}

func (wfa DataMongoAccessor) LoadAll() ([]utils.ShallowDBObject, int, error) {
	objs := []utils.ShallowDBObject{}
	res_mongo, code, err := mongo.MONGOService.LoadAll(wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not retrieve any from db. Error: " + err.Error())
		return nil, code, err
	}
	var results []DataResource
	if err = res_mongo.All(mongo.MngoCtx, &results); err != nil {
		return nil, 404, err
	}
	for _, r := range results {
		objs = append(objs, &r.AbstractResource)
	}
	return objs, 200, nil
}
