package datacenter

import (
	"cloud.o-forge.io/core/oc-lib/dbs/mongo"
	"cloud.o-forge.io/core/oc-lib/models/utils"
)

type DatacenterMongoAccessor struct {
	utils.AbstractAccessor
}

func (dca *DatacenterMongoAccessor) DeleteOne(id string) (utils.DBObject, int, error) {
	return dca.GenericDeleteOne(id, dca)
}

func (dca *DatacenterMongoAccessor) UpdateOne(set utils.DBObject, id string) (utils.DBObject, int, error) {
	return dca.GenericUpdateOne(set, id, dca)
}

func (dca *DatacenterMongoAccessor) StoreOne(data utils.DBObject) (utils.DBObject, int, error) {
	return dca.GenericStoreOne(data, dca)
}

func (dca *DatacenterMongoAccessor) CopyOne(data utils.DBObject) (utils.DBObject, int, error) {
	return dca.GenericStoreOne(data, dca)
}

func (dca *DatacenterMongoAccessor) LoadOne(id string) (utils.DBObject, int, error) {
	var datacenter DatacenterResource

	res_mongo, code, err := mongo.MONGOService.LoadOne(id, dca.GetType())
	if err != nil {
		dca.Logger.Error().Msg("Could not retrieve " + id + " from db. Error: " + err.Error())
		return nil, code, err
	}

	res_mongo.Decode(&datacenter)

	return &datacenter, 200, nil
}

func (wfa DatacenterMongoAccessor) LoadAll() ([]utils.ShallowDBObject, int, error) {
	objs := []utils.ShallowDBObject{}
	res_mongo, code, err := mongo.MONGOService.LoadAll(wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not retrieve any from db. Error: " + err.Error())
		return nil, code, err
	}
	var results []DatacenterResource
	if err = res_mongo.All(mongo.MngoCtx, &results); err != nil {
		return nil, 404, err
	}
	for _, r := range results {
		objs = append(objs, &r.AbstractResource)
	}
	return objs, 200, nil
}
