package oclib

import (
	"cloud.o-forge.io/core/oc-lib/dbs/mongo"
	"cloud.o-forge.io/core/oc-lib/models/utils"
)

type WorkflowResourceMongoAccessor struct {
	utils.AbstractAccessor
}

func (wfa *WorkflowResourceMongoAccessor) DeleteOne(id string) (utils.DBObject, int, error) {
	return wfa.GenericDeleteOne(id, wfa)
}

func (wfa *WorkflowResourceMongoAccessor) UpdateOne(set utils.DBObject, id string) (utils.DBObject, int, error) {
	return wfa.GenericUpdateOne(set, id, wfa)
}

func (wfa *WorkflowResourceMongoAccessor) StoreOne(data utils.DBObject) (utils.DBObject, int, error) {
	return wfa.GenericStoreOne(data, wfa)
}

func (wfa *WorkflowResourceMongoAccessor) CopyOne(data utils.DBObject) (utils.DBObject, int, error) {
	res, _, _ := wfa.LoadOne(data.GetID())
	data.(*WorkflowResource).WorkflowID = data.GetID()
	if res == nil {
		return wfa.GenericStoreOne(data, wfa)
	} else {
		data.(*WorkflowResource).UUID = res.GetID()
		return wfa.GenericUpdateOne(data, res.GetID(), wfa)
	}
}

func (wfa *WorkflowResourceMongoAccessor) LoadOne(id string) (utils.DBObject, int, error) {
	var workflow WorkflowResource
	res_mongo, code, err := mongo.MONGOService.LoadOne(id, wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not retrieve " + id + " from db. Error: " + err.Error())
		return nil, code, err
	}
	res_mongo.Decode(&workflow)
	return &workflow, 200, nil
}

func (wfa WorkflowResourceMongoAccessor) LoadAll() ([]utils.ShallowDBObject, int, error) {
	objs := []utils.ShallowDBObject{}
	res_mongo, code, err := mongo.MONGOService.LoadAll(wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not retrieve any from db. Error: " + err.Error())
		return nil, code, err
	}
	var results []WorkflowResource
	if err = res_mongo.All(mongo.MngoCtx, &results); err != nil {
		return nil, 404, err
	}
	for _, r := range results {
		objs = append(objs, &r.AbstractResource)
	}
	return objs, 200, nil
}
