package oclib

import (
	"errors"

	"cloud.o-forge.io/core/oc-lib/dbs/mongo"
	"cloud.o-forge.io/core/oc-lib/models/utils"
	"cloud.o-forge.io/core/oc-lib/models/workflow_execution"
	"github.com/vk496/cron"
)

type WorkflowMongoAccessor struct {
	utils.AbstractAccessor
}

func (wfa *WorkflowMongoAccessor) DeleteOne(id string) (utils.DBObject, int, error) {
	return wfa.GenericDeleteOne(id, wfa)
}

func (wfa *WorkflowMongoAccessor) execution(realData *Workflow, delete bool) (int, error) {
	if delete {
		mongo.MONGOService.DeleteMultiple(map[string]interface{}{
			"workflow_id": realData.UUID,
		}, wfa.GetType())
	}
	if realData.Schedule != nil {
		accessor := workflow_execution.WorkflowExecutionMongoAccessor{}
		if realData.Schedule.Start.IsZero() {
			return 422, errors.New("should get a start date on the scheduler.")
		}
		if len(realData.Schedule.Cron) > 0 {
			if realData.Schedule.End.IsZero() {
				return 422, errors.New("a cron task should got a end date.")
			}
			c, err := cron.Parse(realData.Schedule.Cron)
			if err != nil {
				return 422, errors.New("Bad cron message: " + err.Error())
			}
			for s := c.Next(realData.Schedule.Start); !s.IsZero() && s.Before(realData.Schedule.End); s = c.Next(s) {
				obj := &workflow_execution.WorkflowExecution{
					ExecDate:   s,
					EndDate:    realData.Schedule.End,
					State:      workflow_execution.SCHEDULED,
					WorkflowID: realData.UUID,
				}
				accessor.StoreOne(obj)
			}

		} else {
			obj := &workflow_execution.WorkflowExecution{
				ExecDate:   realData.Schedule.Start,
				EndDate:    realData.Schedule.End,
				State:      workflow_execution.SCHEDULED,
				WorkflowID: realData.UUID,
			}
			accessor.StoreOne(obj)
		}
	}
	return 200, nil
}

func (wfa *WorkflowMongoAccessor) UpdateOne(set utils.DBObject, id string) (utils.DBObject, int, error) {
	if code, err := wfa.execution(set.(*Workflow), true); err != nil {
		return nil, code, err
	}
	return wfa.GenericUpdateOne(set, id, wfa)
}

func (wfa *WorkflowMongoAccessor) StoreOne(data utils.DBObject) (utils.DBObject, int, error) {
	res, code, err := wfa.GenericStoreOne(data, wfa)
	if err != nil {
		return nil, code, err
	}
	if code, err := wfa.execution(res.(*Workflow), false); err != nil {
		return nil, code, err
	}
	return res, code, err
}

func (wfa *WorkflowMongoAccessor) CopyOne(data utils.DBObject) (utils.DBObject, int, error) {
	return wfa.GenericStoreOne(data, wfa)
}

func (wfa *WorkflowMongoAccessor) LoadOne(id string) (utils.DBObject, int, error) {
	var workflow Workflow
	res_mongo, code, err := mongo.MONGOService.LoadOne(id, wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not retrieve " + id + " from db. Error: " + err.Error())
		return nil, code, err
	}
	res_mongo.Decode(&workflow)
	return &workflow, 200, nil
}

func (wfa WorkflowMongoAccessor) LoadAll() ([]utils.ShallowDBObject, int, error) {
	objs := []utils.ShallowDBObject{}
	res_mongo, code, err := mongo.MONGOService.LoadAll(wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not retrieve any from db. Error: " + err.Error())
		return nil, code, err
	}
	var results []Workflow
	if err = res_mongo.All(mongo.MngoCtx, &results); err != nil {
		return nil, 404, err
	}
	for _, r := range results {
		objs = append(objs, &r.AbstractObject)
	}
	return objs, 200, nil
}
